#pragma once
#include "packet.h"

extern packet_t pA, pB, safe;
extern packet_t *astate, *incoming;
extern comm_state cs;
extern char comm_ok;
extern long last_p;

/*
#define CLOCKWISE 0
#define ANTICLOCKWISE 1
*/

// Drive modes
#define DRIVE_BASIC 0
#define DRIVE_TRANSLATION 1
#define DRIVE_ROTATION 2

// Basic mode conditions, specifies which direction and turning direction the robot is using
#define DRIVE_BASIC_STOP        0
#define DRIVE_BASIC_FORWARD     1
#define DRIVE_BASIC_FRONTLEFT   2
#define DRIVE_BASIC_FRONTRIGHT  3
#define DRIVE_BASIC_BACKWARD    4
#define DRIVE_BASIC_BACKLEFT    5
#define DRIVE_BASIC_BACKRIGHT   6

typedef struct { 
    byte drive_mode = DRIVE_BASIC;
    // byte spin_direction = CLOCKWISE; // Current state, 0 = clockwise, 1 = anticlockwise
    // byte drive_direction = DRIVE_FORWARDS; // Current state, Directions: 0 = forwards, 1 = right, 2 = backwards, 3 = left

    //byte target_spin_direction = CLOCKWISE; // Target state
    //byte target_drive_direction = DRIVE_FORWARDS; // Target state
    
    float target_drive_power = 127.0; // 0 - 127
    float current_drive_power = 127.0; // 0 - 127
    
    float spin_location = 0; // 0 - 360, CURRENT tracked body orientation relative to initial (or reset) orientation
    float spin_offset = 0; // 0 - 360, offset applied to target spin when setting
    
    // CURRENT wheel orientations relative to robot body, read from encoders & convert
    float front_left_current_spin = 0;
    float front_right_current_spin = 0;
    float back_left_current_spin = 0;
    float back_right_current_spin = 0;
    // TARGET wheel orientations relative to robot body
    float front_left_target_spin = 0;
    float front_right_target_spin = 0;
    float back_left_target_spin = 0;
    float back_right_target_spin = 0;
    // Motor power coefficients, this is used when motors must turn at different speeds. This is an input value, and is not directly affected by the current robot conditions
    // Between 0 and 1
    float front_left_coefficient = 0;
    float front_right_coefficient = 0;
    float back_left_coefficient = 0;
    float back_right_coefficient = 0;
    
} swerve_wheel;

#define SerComm Serial1   //Serial port connected to Xbee
#define DIAMOND_LEFT 0
#define DIAMOND_DOWN 1
#define DIAMOND_RIGHT 2
#define DIAMOND_UP 3
#define SHOULDER_TOP_LEFT 4
#define SHOULDER_TOP_RIGHT 5
#define SHOULDER_BOTTOM_LEFT 6
#define SHOULDER_BOTTOM_RIGHT 7
#define SMALL_LEFT 8
#define SMALL_RIGHT 9
//10 and 11 are probably the stick buttons
//but we haven't checked recently
#define DPAD_UP 12
#define DPAD_RIGHT 13
#define DPAD_DOWN 14
#define DPAD_LEFT 15

// pins for motor controller 1 (right)
#define ALI1 0
#define AHI1 1
#define BHI1 2
#define BLI1 3
#define DENABLE1 8
//#define DREADY1 30

// and 2 (left)
#define ALI2 4
#define AHI2 5
#define BHI2 6
#define BLI2 7
#define DENABLE2 9
//#define DREADY2 31

#define try_enable_right(e,VBATT) try_enable_osmc(e,DENABLE1,VBATT,ALI1,BLI1,AHI1,BHI1)
#define try_enable_left(e,VBATT) try_enable_osmc(e,DENABLE2,VBATT,ALI2,BLI2,AHI2,BHI2)
#define drive_right(e,x) drive_osmc(e,DENABLE1,x,0,ALI1,BLI1,AHI1,BHI1)
#define drive_left(e,x) drive_osmc(e,DENABLE2,x,0,ALI2,BLI2,AHI2,BHI2)

#define DEADBAND_HALF_WIDTH 10  // Control input deadband radius
#define FAILTIME 200    //Failsafe timeout in milliseconds
#define DEBUGPRINT(x) SerCommDbg.println(x)
#define SerCommDbg Serial   //Serial port for debugging info
